###########################################################################
#
# HDLList.pm --
# A component of the Greenstone digital library software
# from the New Zealand Digital Library Project at the 
# University of Waikato, New Zealand.
#
# Copyright (C) 1999 New Zealand Digital Library Project
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
###########################################################################


# this is kind of a mutated version of AZList, written
# specifically for the hdl collection. It's the same as
# an AZList classification except it contains a Hierarcical
# 'Magazines' classification as well as the standard 
# alphabetical ones

# There's also no need to provide a metadata element
# to sort by - it always uses Title

# magazines are sorted by filename

package HDLList;

use sorttools;

sub new {
    my ($class) = @_;

    return bless {
	'list'=>{},
	'magclassifier'=>""
	}, $class;
}

sub init {
    my $self = shift (@_);

    $self->{'list'} = {};

    # load up a Hierarchy classifier for the 'magazines' sub-section
    my $hrname = &util::filename_cat($ENV{'GSDLHOME'},"perllib/classify/Hierarchy.pm"); 
    if (-e $hrname) { require $hrname; }
    else { die "HDLList ERROR - couldn't find classifier \"$hrname\"\n"; }

    # create the Hierarchy classifier object
    my ($hrclassobj);
    eval ("\$hrclassobj = new Hierarchy(\"hfile=mags.txt\", \"metadata=Magazine\", \"sort=Filename\", \"title=_magazines_\")");
    die "$@" if $@;

    $hrclassobj->init();
    $self->{'magclassifier'} = $hrclassobj;
}

sub classify {
    my $self = shift (@_);
    my ($doc_obj) = @_;

    my $doc_OID = $doc_obj->get_OID();

    my $magazine = $doc_obj->get_metadata_element ($doc_obj->get_top_section(),
						   'Magazine');

    my $title = $doc_obj->get_metadata_element ($doc_obj->get_top_section(),
						'Title');

    # if this document contains <Magazine> metadata we want to shove it into
    # the Hierarchical magazine classification (and not include it in the
    # alphabetical list), otherwise shove it in the AZList bit (as long as
    # it has a <Title>
    if (defined $magazine && $magazine =~ /\w/) {
	$self->{'magclassifier'}->classify ($doc_obj);

    } elsif (defined $title && $title =~ /\w/) {
	&sorttools::format_string_english (\$title);
	
	if (defined $self->{'list'}->{$doc_OID}) {
	    print STDERR "WARNING: HDLList::classify called multiple times for $doc_OID\n";
	} 
	$self->{'list'}->{$doc_OID} = $title;
    }
}


sub get_classify_info {
    my $self = shift (@_);

    my @classlist = sort {$self->{'list'}->{$a} cmp $self->{'list'}->{$b};} keys %{$self->{'list'}};

    my $classifyinfo = $self->splitlist (\@classlist);

    # shove in the magazine bit
    push (@{$classifyinfo->{'contains'}}, $self->{'magclassifier'}->get_classify_info());
    
    return $classifyinfo;
}


sub get_entry {
    my $self = shift (@_);
    my ($title, $childtype, $thistype) = @_;
    
    # organise into classification structure
    my %classifyinfo = ('childtype'=>$childtype,
			'Title'=>$title,
			'contains'=>[]);
    $classifyinfo{'thistype'} = $thistype 
	if defined $thistype && $thistype =~ /\w/;

    return \%classifyinfo;
}

# splitlist takes an ordered list of classifications (@$classlistref) and splits it
# up into alphabetical sub-sections.
sub splitlist {
    my $self = shift (@_);
    my ($classlistref) = @_;
    my $classhash = {};

    # top level
    my $classifyinfo = $self->get_entry ("Title", "HList", "Invisible");

    # don't need to do any splitting if there are less than 20 classifications
    # we have a A-Z section containing everything if there aren't many 
    # classifications (since we've already got a 'Magazines' section)
    if ((scalar @$classlistref) <= 20) {
	my $allinfo = $self->get_entry ("A-Z", "VList");
	foreach $subOID (@$classlistref) {
	    push (@{$allinfo->{'contains'}}, {'OID'=>$subOID});
	}
	push (@{$classifyinfo->{'contains'}}, $allinfo);
	return $classifyinfo;
    }
	
    # first split up the list into separate A-Z and 0-9 classifications
    foreach $classification (@$classlistref) {
	my $title = $self->{'list'}->{$classification};
	$title =~ s/^(.).*$/$1/;
	$title =~ tr/[a-z]/[A-Z]/;
	if ($title =~ /^[0-9]$/) {$title = '0-9';}
	elsif ($title !~ /^[A-Z]$/) {
	    print STDERR "HDLList: WARNING $classification has badly formatted title ($title)\n";
	}
	$classhash->{$title} = [] unless defined $classhash->{$title};
	push (@{$classhash->{$title}}, $classification);
    }
    $classhash = $self->compactlist ($classhash);

    my @tmparr = ();
    foreach $subsection (sort keys (%$classhash)) {
	push (@tmparr, $subsection);
    }
    
    # if there's a 0-9 section it will have been sorted to the beginning
    # but we want it at the end
    if ($tmparr[0] eq '0-9') {
	shift @tmparr;
	push (@tmparr, '0-9');
    }

    foreach $subclass (@tmparr) {
	my $tempclassify = $self->get_entry($subclass, "VList");
	foreach $subsubOID (@{$classhash->{$subclass}}) {
	    push (@{$tempclassify->{'contains'}}, {'OID'=>$subsubOID});
	}
	push (@{$classifyinfo->{'contains'}}, $tempclassify);
    }

    return $classifyinfo;
}

sub compactlist {
    my $self = shift (@_);
    my ($classhashref) = @_;
    my $compactedhash = {};
    my @currentOIDs = ();
    my $currentfirstletter = "";
    my $currentlastletter = "";
    my $lastkey = "";

    # minimum and maximum documents to be displayed per page.
    # the actual maximum will be max + (min-1).
    # the smallest sub-section is a single letter at present
    # so in this case there may be many times max documents
    # displayed on a page.
    my $min = 10; 
    my $max = 30;

    foreach $subsection (sort keys %$classhashref) {
	if ($subsection eq '0-9') {
	    @{$compactedhash->{$subsection}} = @{$classhashref->{$subsection}};
	    next;
	}
	$currentfirstletter = $subsection if $currentfirstletter eq "";
	if ((scalar (@currentOIDs) < $min) ||
	    ((scalar (@currentOIDs) + scalar (@{$classhashref->{$subsection}})) <= $max)) {
	    push (@currentOIDs, @{$classhashref->{$subsection}});
	    $currentlastletter = $subsection;
	} else {

	    if ($currentfirstletter eq $currentlastletter) {
		@{$compactedhash->{$currentfirstletter}} = @currentOIDs;
		$lastkey = $currentfirstletter;
	    } else {
		@{$compactedhash->{"$currentfirstletter-$currentlastletter"}} = @currentOIDs;
		$lastkey = "$currentfirstletter-$currentlastletter";
	    } 
	    if (scalar (@{$classhashref->{$subsection}}) >= $max) {
		$compactedhash->{$subsection} = $classhashref->{$subsection};
		@currentOIDs = ();
		$currentfirstletter = "";
	    } else {
		@currentOIDs = @{$classhashref->{$subsection}};
		$currentfirstletter = $subsection;
		$currentlastletter = $subsection;
	    }
	}
    }

    # add final OIDs to last sub-classification if there aren't many otherwise
    # add final sub-classification
    if (scalar (@currentOIDs) < $min) {
	my ($newkey) = $lastkey =~ /^(.)/;
	@currentOIDs = (@{$compactedhash->{$lastkey}}, @currentOIDs);
	delete $compactedhash->{$lastkey};
	@{$compactedhash->{"$newkey-$currentlastletter"}} = @currentOIDs;	
    } else {
	if ($currentfirstletter eq $currentlastletter) {
	    @{$compactedhash->{$currentfirstletter}} = @currentOIDs;
	} else {
	    @{$compactedhash->{"$currentfirstletter-$currentlastletter"}} = @currentOIDs;
	} 
    }

    return $compactedhash;
}

1;
